import { AnyAction } from 'redux';
import { FxSlice, FxState } from './FxState';
import { Effect } from './effects';
export declare type FxActionType = 'call' | 'data' | 'error' | 'complete' | 'destroy';
export interface FxMeta {
    effectName: string;
    fxType: FxActionType;
    id: string;
}
export interface FxAction<Payload extends any[]> extends AnyAction {
    type: string;
    payload: Payload;
    fx: FxMeta;
}
export declare const isFxAction: <Payload extends any[]>(action: AnyAction) => action is FxAction<Payload>;
export declare const isCallFxAction: (action: AnyAction) => action is FxAction<any>;
export interface BaseFxActionCreator<Payload extends any[]> {
    readonly id: string;
    readonly type: string;
    readonly fxType: FxActionType;
    readonly effectName: string;
    readonly match: (action: AnyAction) => action is FxAction<Payload>;
}
export interface FxActionCreator<Payload extends any[]> extends BaseFxActionCreator<Payload> {
    (...payload: Payload): FxAction<Payload>;
}
export interface FxActionCreators<Data, Params extends any[]> {
    readonly id: string;
    readonly effectName: string;
    readonly call: FxActionCreator<Params>;
    readonly data: FxActionCreator<[Data]>;
    readonly error: FxActionCreator<[any]>;
    readonly complete: FxActionCreator<[]>;
    readonly destroy: FxActionCreator<[]>;
    readonly selector: (state: FxSlice) => FxState<Data, Params>;
}
export interface FxActionsConfig<Data, Params extends any[]> {
    effect: Effect<Data, Params>;
    effectName?: string;
}
export declare const fxActions: <Data, Params extends any[]>(effectOrConfig: import("./effects").ObservableInputEffect<Data, Params> | import("./effects").AsyncGeneratorEffect<Data, Params> | FxActionsConfig<Data, Params>) => FxActionCreators<Data, Params>;
