"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var uuid_1 = require("uuid");
var FxState_1 = require("./FxState");
var effects_1 = require("./effects");
var cache_1 = require("./cache");
exports.isFxAction = function (action) {
    return 'fx' in action;
};
exports.isCallFxAction = function (action) {
    return exports.isFxAction(action) && action.fx.fxType === 'call';
};
function getConfig(effect, id) {
    var config = typeof effect === 'function'
        ? {
            effect: effect,
            effectName: effect.name,
        }
        : __assign({}, effect, { effectName: effect.effectName || effect.effect.name });
    if (!config.effectName) {
        // tslint:disable-next-line:no-console
        console.warn('fx-state:\n', '#fxActions was called with an anonymous side-effect function, and no `effectName` was provided.\n', 'The action type will be derived using the FxActions uuid, which is ugly and non-descriptive.\n', 'It is recommended that you either provide a non-anonymous Effect or provide a `effectName`.');
        config.effectName = id;
    }
    return config;
}
var fxacf = function (id, effectName, fxType) {
    var type = "fx/" + effectName + "/" + fxType;
    var actionCreator = function () {
        var payload = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            payload[_i] = arguments[_i];
        }
        return ({
            type: type,
            payload: payload,
            fx: { effectName: effectName, fxType: fxType, id: id },
        });
    };
    var match = function (action) {
        return exports.isFxAction(action) &&
            action.fx.effectName === effectName &&
            action.fx.fxType === fxType &&
            // really, only checking the id is actually necessary
            action.fx.id === id;
    };
    return Object.freeze(Object.assign(actionCreator, {
        id: id,
        type: type,
        effectName: effectName,
        fxType: fxType,
        match: match,
    }));
};
exports.fxActions = function (effectOrConfig) {
    var id = uuid_1.v4();
    var _a = getConfig(effectOrConfig, id), effect = _a.effect, effectName = _a.effectName;
    var actions = Object.freeze({
        id: id,
        effectName: effectName,
        call: fxacf(id, effectName, 'call'),
        data: fxacf(id, effectName, 'data'),
        error: fxacf(id, effectName, 'error'),
        complete: fxacf(id, effectName, 'complete'),
        destroy: fxacf(id, effectName, 'destroy'),
        selector: function (state) { return state.fx[id] || FxState_1.initialFxState; },
    });
    cache_1.set(id, actions, effects_1.$fromEffect(effect));
    return actions;
};
//# sourceMappingURL=actions.js.map