import { Reducer, AnyAction } from 'redux';
/**
 * "lifts" a reducer that manages a state shape into a "slice" reducer.
 *
 * For example:
 * ```
 * type A: { someProp: string; otherProp: boolean }
 * const reducerOfA: Reducer<A> = ...
 * const sliceReducerOfA = createSliceReducer({
 *   a: reducerOfA
 * })
 *
 * // typeof sliceReducerOfA: Reducer<{ a: A }>
 * ```
 */
declare const createSliceReducer: <State extends {
    [k: string]: any;
}>(reducerDict: { [K in keyof State]: Reducer<State[K], AnyAction>; }) => Reducer<State, AnyAction>;
export default createSliceReducer;
