"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * "lifts" a reducer that manages a state shape into a "slice" reducer.
 *
 * For example:
 * ```
 * type A: { someProp: string; otherProp: boolean }
 * const reducerOfA: Reducer<A> = ...
 * const sliceReducerOfA = createSliceReducer({
 *   a: reducerOfA
 * })
 *
 * // typeof sliceReducerOfA: Reducer<{ a: A }>
 * ```
 */
var createSliceReducer = function (reducerDict) { return function (state, action) {
    if (state === void 0) { state = {}; }
    return Object.keys(reducerDict).reduce(function (rootState, ns) {
        var _a;
        var newState = reducerDict[ns](rootState && rootState[ns], action);
        return newState !== (state && state[ns])
            ? __assign({}, rootState, (_a = {}, _a[ns] = newState, _a)) : state;
    }, state);
}; };
exports.default = createSliceReducer;
//# sourceMappingURL=createSliceReducer.js.map