"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var chai = require("chai");
var sinon = require("sinon");
var rxjs_1 = require("rxjs");
var redux_1 = require("redux");
require("mocha");
var expect = chai.expect;
var actions_1 = require("./actions");
var middleware_1 = require("./middleware");
describe('fx-state', function () {
    var param1 = 'abc';
    var param2 = 123;
    var data = { prop1: true, prop2: 10 };
    var error = 'error';
    describe('fxEpic', function () {
        var fakeEffectSubject$;
        var effect;
        var actions;
        var action$;
        var action$out;
        beforeEach(function () {
            effect = sinon.spy(function (_1, _2) {
                fakeEffectSubject$ = new rxjs_1.Subject();
                sinon.spy(fakeEffectSubject$, 'subscribe');
                return fakeEffectSubject$;
            });
            actions = actions_1.fxActions(effect);
            action$ = new rxjs_1.Subject();
            action$out = middleware_1.fxEpic(action$);
        });
        describe('#call action', function () {
            describe('given an ObservableFactory', function () {
                it('should call a given ObservableFactory and subscribe to the resulting Observable', function () {
                    action$out.subscribe();
                    action$.next(actions.call(param1, param2));
                    expect(effect).to.have.been.calledWith(param1, param2);
                    expect(fakeEffectSubject$.subscribe).to.have.been.called;
                });
                it('should unsubscribe from the previous Observable when there is a new call action', function () {
                    var subscription = sinon.spy();
                    action$out.subscribe(subscription);
                    action$.next(actions.call(param1, param2));
                    var origSubject = fakeEffectSubject$;
                    fakeEffectSubject$.next(data);
                    expect(subscription).to.have.been.calledWithMatch(actions.data(data));
                    action$.next(actions.call(param1, param2));
                    origSubject.next(data);
                    expect(subscription).not.to.have.been.calledTwice;
                });
                it('should dispatch actions for a subsequent call action', function () {
                    var subscription = sinon.spy();
                    action$out.subscribe(subscription);
                    action$.next(actions.call(param1, param2));
                    action$.next(actions.call(param1, param2));
                    fakeEffectSubject$.next(data);
                    expect(subscription).not.to.have.been.calledTwice;
                });
            });
        });
        describe('observable$', function () {
            describe('#data', function () {
                it('should dispatch a `data` action with the payload sent by the Observable', function () {
                    var subscription = sinon.spy();
                    action$out.subscribe(subscription);
                    action$.next(actions.call(param1, param2));
                    fakeEffectSubject$.next(data);
                    expect(subscription).to.have.been.calledWithMatch(actions.data(data));
                });
            });
            describe('#error', function () {
                it('should dispatch an `error` action with the payload sent by the Observable', function () {
                    var subscription = sinon.spy();
                    action$out.subscribe(subscription);
                    action$.next(actions.call(param1, param2));
                    fakeEffectSubject$.error(error);
                    expect(subscription).to.have.been.calledWithMatch(actions.error(error));
                });
            });
            describe('#complete', function () {
                it('should dispatch a `complete` action', function () {
                    var subscription = sinon.spy();
                    action$out.subscribe(subscription);
                    action$.next(actions.call(param1, param2));
                    fakeEffectSubject$.complete();
                    expect(subscription).to.have.been.calledWithMatch(actions.complete());
                });
            });
        });
        describe('#destroy action', function () {
            it('should unsubscribe from the Observable', function () {
                var subscription = sinon.spy();
                action$out.subscribe(subscription);
                action$.next(actions.call(param1, param2));
                fakeEffectSubject$.next(data);
                expect(subscription).to.have.been.calledWithMatch(actions.data(data));
                action$.next(actions.destroy());
                fakeEffectSubject$.next(data);
                expect(subscription).not.to.have.been.calledTwice;
            });
        });
    });
    describe('fxMiddleware', function () {
        var reducer;
        var store;
        var effect;
        var actions;
        var resolve;
        var reject;
        var callAction;
        beforeEach(function () {
            reducer = sinon.spy(function (_state, _action) { return ({}); });
            store = redux_1.createStore(reducer, {}, redux_1.applyMiddleware(middleware_1.fxMiddleware));
            effect = sinon.spy(function (_p) { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    return [2 /*return*/, new Promise(function (res, rej) {
                            resolve = res;
                            reject = rej;
                        })];
                });
            }); });
            actions = actions_1.fxActions(effect);
            callAction = actions.call(param1, param2);
            store.dispatch(callAction);
        });
        describe('dispatch #call action', function () {
            it('should call the reducer', function () {
                expect(reducer).to.have.been.calledWithMatch({}, callAction);
            });
            it('should call the effect', function () {
                expect(effect).to.have.been.calledWithMatch(param1, param2);
            });
        });
        describe('effect resolves', function () {
            it('should dispatch a data() action, followed by a complete() action', function (done) {
                var expectedActions = [actions.data(data), actions.complete()];
                store.subscribe(function () {
                    expect(reducer).to.have.been.calledWithMatch({}, expectedActions.shift());
                    if (!expectedActions.length) {
                        done();
                    }
                });
                resolve(data);
            });
        });
        describe('effect is rejected', function () {
            it('should dispatch an error() action', function (done) {
                store.subscribe(function () {
                    expect(reducer).to.have.been.calledWithMatch({}, actions.error(error));
                    done();
                });
                reject(error);
            });
        });
    });
});
//# sourceMappingURL=middleware.spec.js.map