"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
function sliceReducers() {
    var reducers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        reducers[_i] = arguments[_i];
    }
    return function (state, action) {
        return state === undefined
            ? // initialize state for each slice reducer with "undefined"
                reducers.reduce(function (accumulatedInitialState, nextReducer) {
                    return reduceAndMergeAfterChecking(nextReducer, undefined, action, accumulatedInitialState);
                }, {})
            : // give each slice reducer the same "current state"
                reducers.reduce(function (accumulatedNextState, nextReducer) {
                    return reduceAndMergeAfterChecking(nextReducer, state, action, accumulatedNextState);
                }, state);
    };
}
exports.default = sliceReducers;
function reduceAndMergeAfterChecking(nextReducer, currentState, action, accumulatedState) {
    var nextStateToMerge = nextReducer(currentState, action);
    for (var _i = 0, _a = Object.keys(nextStateToMerge); _i < _a.length; _i++) {
        var key = _a[_i];
        if (key in accumulatedState &&
            accumulatedState[key] !== nextStateToMerge[key]) {
            // tslint:disable-next-line:no-console
            console.warn('sliceReducers:\n', 'A reducer was given to `sliceReducers` that modifies the same subset of the state', 'tree as a previous reducer. This is not recommended, as the states are merged', 'destructively, so the state returned by the subsequent reducer will always override', 'the state of the previous.\n', 'If you have multiple reducers that are intended to share management of the same slice', 'of a state tree, consider using `chainReducers` instead.');
        }
    }
    return __assign({}, accumulatedState, nextStateToMerge);
}
//# sourceMappingURL=sliceReducers.js.map